package com.bcxin.sync.service.tenant;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bcxin.sync.common.utils.HuaweiIvmUtil;
import com.bcxin.sync.common.utils.IdGeneratorSnowflake;
import com.bcxin.sync.common.utils.PageUtils;
import com.bcxin.sync.configs.SyncConfig;
import com.bcxin.sync.dao.mapper.tenant.DeviceRecordMapper;
import com.bcxin.sync.dtos.request.DeviceRecordListRequest;
import com.bcxin.sync.dtos.response.ExamDeviceResponse;
import com.bcxin.sync.entity.tenant.DeviceRecordEntity;
import com.bcxin.sync.service.RedisCache;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * description：设备录像表
 * author：linchunpeng
 * date：2025/5/28
 */
@Slf4j
@Service
public class DeviceRecordService extends ServiceImpl<DeviceRecordMapper, DeviceRecordEntity> {

    @Autowired
    private SyncConfig syncConfig;
    @Autowired
    private RedisCache redisCache;
    @Autowired
    private IdGeneratorSnowflake snowflake;

    public List<DeviceRecordEntity> getList(DeviceRecordListRequest request) {
        LambdaQueryChainWrapper<DeviceRecordEntity> lqw = this.lambdaQuery();
        if (StringUtils.isNotBlank(request.getDeviceNo())) {
            lqw.like(DeviceRecordEntity::getDeviceNo, request.getDeviceNo());
        }
        if (StringUtils.isNotBlank(request.getDeviceName())) {
            lqw.like(DeviceRecordEntity::getDeviceName, request.getDeviceName());
        }
        if (StringUtils.isNotBlank(request.getBindPlace())) {
            lqw.like(DeviceRecordEntity::getBindPlace, request.getBindPlace());
        }
        if (StringUtils.isNotBlank(request.getStartTimeStart())) {
            lqw.ge(DeviceRecordEntity::getStartTime, DateUtil.parseDateTime(request.getStartTimeStart()));
        }
        if (StringUtils.isNotBlank(request.getStartTimeEnd())) {
            lqw.le(DeviceRecordEntity::getStartTime, DateUtil.parseDateTime(request.getStartTimeEnd()));
        }
        PageUtils.startPage(request);//分页
        return lqw.list();
    }

    /**
     * description：开启华为云设备录像
     * author：linchunpeng
     * date：2025/5/29
     */
    public void openHwDeviceRecord() {
        log.info("设置今天的录像计划");
        Date now = new Date();
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("startTime", DateUtil.beginOfDay(now));
        paramMap.put("endTime", DateUtil.endOfDay(now));
        log.info("时间参数：{}", JSONObject.toJSONString(paramMap));
        List<ExamDeviceResponse> examDeviceList = this.baseMapper.selectExamDeviceList(paramMap);
        if (CollectionUtil.isNotEmpty(examDeviceList)) {
            List<DeviceRecordEntity> entityList = new ArrayList<>();
            log.info("存在有关联考场的设备，考试数量：{}", examDeviceList.size());
            String accessToken = HuaweiIvmUtil.getAccessToken(syncConfig.getHuaweiIvm().getHost(), syncConfig.getHuaweiIvm().getUserId(), syncConfig.getHuaweiIvm().getAk(), syncConfig.getHuaweiIvm().getSk(), redisCache);

            Map<String, List<ExamDeviceResponse>> examDeviceListMap = examDeviceList.stream().collect(Collectors.groupingBy(ExamDeviceResponse::getDeviceNo));
            log.info("存在有关联考场的设备，设备数量：{}", examDeviceListMap.size());

            for (Map.Entry<String, List<ExamDeviceResponse>> entry : examDeviceListMap.entrySet()) {
                String deviceNo = entry.getKey();
                log.info("开始设置华为云设备录像计划，设备编号：{}", deviceNo);
                List<Date> startTimeList = new ArrayList<>();
                List<Date> endTimeList = new ArrayList<>();
                for (ExamDeviceResponse examDeviceResponse : entry.getValue()) {
                    startTimeList.add(examDeviceResponse.getStartTime());
                    endTimeList.add(examDeviceResponse.getEndTime());
                }
                //录像计划的时间点字符串，用来判断路线时间是否有有变化
                String recordTimeValue = startTimeList.stream().map(DateUtil::formatDateTime).collect(Collectors.joining(";"))
                        + "|" + endTimeList.stream().map(DateUtil::formatDateTime).collect(Collectors.joining(";"));
                log.info("开始设置华为云设备录像计划，录像计划的时间点字符串：{}", recordTimeValue);
                //看看今天的录像计划是否有变，如果没有变，就不需要重新设置了
                String timeKey = deviceNo.concat("_").concat(DateUtil.formatDate(now));
                String timeValue = redisCache.getCacheObject(timeKey);
                if (StringUtils.isBlank(timeValue) || !recordTimeValue.equals(timeValue)) {
                    log.info("缓存不存在 || 缓存的值和新的值不一致，需要重新设置监控的录像计划");
                    HuaweiIvmUtil.setDeviceRecordPlan(syncConfig.getHuaweiIvm().getHost(), syncConfig.getHuaweiIvm().getUserId(),
                            accessToken, deviceNo, startTimeList, endTimeList);

                    log.info("删除录像记录");
                    LambdaQueryChainWrapper<DeviceRecordEntity> lqw = this.lambdaQuery();
                    lqw.eq(DeviceRecordEntity::getDeviceNo, deviceNo);
                    this.baseMapper.delete(lqw);

                    log.info("新增录像记录");
                    for (ExamDeviceResponse examDeviceResponse : entry.getValue()) {
                        DeviceRecordEntity entity = new DeviceRecordEntity();
                        entity.setId(snowflake.snowflakeId());
                        entity.setDeviceNo(deviceNo);
                        entity.setDeviceName(examDeviceResponse.getDeviceName());
                        entity.setDeviceType(examDeviceResponse.getDeviceType());
                        entity.setBindType(examDeviceResponse.getBindType());
                        entity.setBindPlace(examDeviceResponse.getBindPlace());
                        entity.setStartTime(examDeviceResponse.getStartTime());
                        entity.setEndTime(examDeviceResponse.getEndTime());
                        entity.setDurationTime((int)DateUtil.between(examDeviceResponse.getStartTime(), examDeviceResponse.getEndTime(), DateUnit.SECOND, true));
                        entity.setCreateTime(now);
                        entity.setUpdateTime(now);
                        entityList.add(entity);
                    }
                    //放入缓存
                    redisCache.setCacheObject(timeKey, recordTimeValue, 1, TimeUnit.DAYS);
                }
            }

            if (CollectionUtil.isNotEmpty(entityList)) {
                log.info("保存入库");
                this.saveBatch(entityList);
            }
        }
    }

}